import java.util.ArrayList;

/**
 * A utility class containing methods for sorting arrays
 * 
 * @author Joel Ross
 * @version Fa2014
 */
@SuppressWarnings({"unchecked","rawtypes"}) //So we can easily work with arrays of Comparables
public class SearchSorts
{
	
	/**
	 * Uses a linear search to find the given item in the list. Only works on Comparables for consistency
	 * @param list A list of items to search
	 * @param target The target to search for
	 * @return The index of the target in the list, or -1 if the target is not in the list
	 */
	public static int linearSearch(Comparable[] list, Comparable target)
	{
		for(int i=0; i<list.length; i++)
		{
			if(list[i].compareTo(target) == 0)
				return i;
		}
		return -1;
	}

	
	/**
	 * Uses a linear search to find the given item in the list. The list must be sorted for this method to return correctly!
	 * @param list A list of items to search
	 * @param target The target to search for
	 * @return The index of the target in the list, or -1 if the target is not in the list
	 */
	public static int binarySearch(Comparable[] list, Comparable target)
	{
		int start = 0;
		int end = list.length;
		while(end - start > 1)
		{
			int middle = (start+end)/2;
			if(target.compareTo(list[middle]) == 0)
				return middle;
			else if(target.compareTo(list[middle]) < 0)
				end = middle;
			else
				start = middle;
		}
		if(target.compareTo(list[start]) == 0)
			return start;
		else
			return -1;
	}

		
	/**
	 * Uses bubble sort to sort the provided list (in place)
	 * @param list The list to sort
	 */
	public static void bubbleSort(Comparable[] list)
	{
		boolean sorted;
		do
		{
			sorted = true;
			for(int i=0; i<list.length-1; i++)
			{
				if(list[i].compareTo(list[i+1]) > 0)
				{
					Comparable tmp = list[i];
					list[i] = list[i+1];
					list[i+1] = tmp;
					sorted = false;
				}
			}

		}
		while(!sorted);
	}

	
	/**
	 * Uses selection sort to sort the provided list (in place)
	 * @param list The list to sort
	 */
	public static void selectionSort(Comparable[] list)
	{
		for(int i=0; i<list.length-1; i++)
		{
			int selected = i;
			for(int j=i+1; j<list.length; j++)
			{
				if(list[j].compareTo(list[selected]) < 0)
				{
					selected = j;
				}
			}
			Comparable tmp = list[i];
			list[i] = list[selected];
			list[selected] = tmp;
		}
	}


	/**
	 * Uses insertion sort to sort the provided list (in place)
	 * @param list The list to sort
	 */
	public static void insertionSort(Comparable[] list)
	{
		for(int i=1; i<list.length; i++)
		{
			Comparable inserted = list[i];
			int hole = i;
			while(hole > 0 && inserted.compareTo(list[hole-1]) < 0)
			{
				list[hole] = list[hole-1];
				hole--;
			}
			list[hole] = inserted;
		}
	}
	
}
